from typing import Dict, Any

from simple_mailer.config import settings
from simple_mailer.dispatcher import Dispatcher
from simple_mailer.exceptions import BaseSimpleMailerException

try:
    import pkg_resources
except ImportError:
    pkg_resources = None  # type: ignore


def get_version() -> str:
    """Get the version designation for this software distribution"""
    if pkg_resources:
        return pkg_resources.get_distribution("simple_mailer").version
    else:
        return "unknown"


def get_env_variables() -> Dict[str, Any]:
    """Get all the related environment variables and their values"""
    keys = settings.get_defaults().keys()
    return {key: settings[key] for key in keys}


def get_smtp_connection() -> Dict[str, str]:
    """Get information about the smtpd connection"""
    try:
        Dispatcher().get_server().connect().disconnect()
    except (BaseSimpleMailerException, OSError) as exc:
        return {"status": "error", "reason": f"{exc}"}
    else:
        return {"status": "OK"}



import re
from typing import Dict, Any

_sensitive_rxp = re.compile(r"^\w+(_SECRET|_PASSWD)$")


def cloak(ns: Dict[str, Any], stub="********") -> Dict:
    """Cloak sensitive variables to not reveal secrets"""
    cloaked_ns = {}
    for key, val in ns.items():
        if val and _sensitive_rxp.match(key):
            cloaked_ns[key] = stub
        else:
            cloaked_ns[key] = val
    return cloaked_ns



import asyncore
import time
from smtpd import SMTPChannel, SMTPServer
from threading import Thread
from types import SimpleNamespace
from typing import List, Optional


class CustomSMTPChannel(SMTPChannel):
    def smtp_AUTH(self, arg):
        self.push("235 2.7.0 Authentication successful")


class SMTPServerThread(Thread):
    def __init__(self):
        super().__init__()
        self.host_port: Optional[int] = None
        self.sent_mail: list = []

    def run(self):
        server_thread = self

        class _SMTPServer(SMTPServer):
            channel_class = CustomSMTPChannel

            def process_message(
                self,
                _: tuple,
                sender: str,
                recipients: List[str],
                msg: bytes,
                *args,
                **kwargs,
            ):
                server_thread.sent_mail.append(
                    SimpleNamespace(
                        **{"from": sender, "to": recipients, "body": msg}
                    )
                )

        self.smtp = _SMTPServer(("localhost", 0), None)
        self.host_port = self.smtp.socket.getsockname()
        asyncore.loop(timeout=0.1)

    def close(self):
        self.smtp.close()


class SMTPServerFixture:
    def __init__(self):
        self._thread = SMTPServerThread()
        self._thread.start()

    @property
    def host_port(self):
        """SMTP server's listening address as a (host, port) tuple"""
        while self._thread.host_port is None:
            time.sleep(0.1)
        return self._thread.host_port

    @property
    def host(self):
        return self.host_port[0]

    @property
    def port(self):
        return self.host_port[1]

    def close(self):
        self._thread.sent_mail.clear()
        self._thread.close()
        self._thread.join(10)
        if self._thread.is_alive():
            raise RuntimeError("smtp server thread did not stop in 10 sec")

    @property
    def sent_mail(self) -> list:
        return self._thread.sent_mail



#!/usr/bin/env python

from setuptools import setup


def long_description():
    with open('README.md') as fd:
        return fd.read()


setup(
    name='simple-mailer',
    version='0.13.1',
    author='Rigel Di Scala',
    author_email='zedr@zedr.com',
    description='A simple mailer for web forms',
    license='MIT',
    long_description=long_description(),
    long_description_content_type='text/markdown',
    url='https://github.com/zedr/simple-mailer',
    install_requires=[
        'bottle==0.12.18',
        'lxml==4.5.0',
        'jinja2==2.11.2'
    ],
    package_dir={'': 'src'},
    packages=['simple_mailer'],
    py_modules=['simple_mailer'],
    include_package_data=True,
    entry_points={
        'console_scripts': ['simple-mailer=simple_mailer.web:run_application']
    },
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.7'
)



import io
import json
import os

import bottle
import pytest
from simple_mailer.config import settings
from simple_mailer.tests.fixtures.smtpd import SMTPServerFixture
from simple_mailer.web import get_application
from webtest import TestApp


@pytest.fixture(scope="session")
def smtpd(request):
    fixture = SMTPServerFixture()
    os.environ.setdefault("SMTP_PORT", str(fixture.port))
    os.environ.setdefault("USE_TLS", "false")
    os.environ.setdefault("FROM_ADDRESS", "me@example.com")
    os.environ.setdefault("TO_ADDRESS", "you@example.com")
    request.addfinalizer(fixture.close)
    return fixture


@pytest.fixture
def test_http_client():
    return TestApp(get_application())


@pytest.fixture(autouse=True)
def reset_smtpd_storage(smtpd):
    smtpd.sent_mail.clear()


@pytest.fixture(scope="module")
def urlencoded_post_request():
    body_obj = io.BytesIO(b"email=me%40example.com&subscribe_me=True")
    fixture = bottle.Request(
        environ={
            "REQUEST_METHOD": "POST",
            "PATH_INFO": settings.MAILER_PATH,
            "CONTENT_TYPE": "application/x-www-form-urlencoded",
            "CONTENT_LENGTH": 40,
            "bottle.request.body": body_obj,
        }
    )
    return fixture


@pytest.fixture(scope="module")
def json_post_request():
    body_obj = io.BytesIO(
        json.dumps({"email": "me@example.org", "message": "hello!"})
    )
    fixture = bottle.Request(
        environ={
            "REQUEST_METHOD": "POST",
            "PATH_INFO": settings.MAILER_PATH,
            "CONTENT_TYPE": "application/json",
            "CONTENT_LENGTH": 40,
            "bottle.request.body": body_obj,
        }
    )
    return fixture


@pytest.fixture
def captcha_server(request):
    os.environ.setdefault("CAPTCHA", "recaptchav3")
    recaptcha_site_key = "fIGZeiSgiRJDMMexWJmwAsMHAuWjwbcUwKvsgOAj"
    os.environ.setdefault("RECAPTCHAV3_SITE_KEY", recaptcha_site_key)

    def fin():
        os.environ.pop("CAPTCHA", None)
        os.environ.pop("RECAPTCHAV3_SITE_KEY", None)

    request.addfinalizer(fin)
    return {"key": recaptcha_site_key}


@pytest.fixture
def mocked_https_client(request):
    import http.client

    class MockedHTTPResponse:
        status = 200

        def read(self):
            return json.dumps({"success": True})

    class MockedHTTPSConnection:
        def __init__(self, host):
            self.host = host

        def request(self, method, path, params, headers):
            pass

        def getresponse(self):
            return MockedHTTPResponse()

    _HTTPSConnection = http.client.HTTPSConnection
    http.client.HTTPSConnection = MockedHTTPSConnection

    def fin():
        http.client.HTTPSConnection = _HTTPSConnection

    request.addfinalizer(fin)
    return None
